@@ -1,7 +1,3 @@
-from typing import IO, Any, Callable, Dict, List, Optional, Sequence, Tuple, Type, Union
-
-Dict, Type
-
 import argparse
 import datetime
 import difflib
@@ -13,6 +9,7 @@ import os
 import sys
 import time
 import traceback
+from typing import IO, Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 
 import colorama
 import tabulate
@@ -40,7 +37,12 @@ _MESSAGES = list()
 # and over again:
 _ENABLE_XTERM_TITLE = None
 
-colorama.init()
+
+if os.name == "nt":
+    # On Windows using `isatty()` does *not* work reliably,
+    # so we always setup colorama.
+    # See colors_enabled() for details
+    colorama.init()
 
 
 # Tokens can be strings, or Color, UnicodeSequence or Symbol instances,
@@ -56,7 +58,7 @@ def setup(
     title: str = "auto",
     timestamp: bool = False,
 ) -> None:
-    """ Configure behavior of message functions.
+    """Configure behavior of message functions.
 
     :param verbose: Whether :func:`debug` messages should get printed
     :param quiet: Hide every message except :func:`warning`, :func:`error`, and
@@ -75,7 +77,7 @@ def _setup(**kwargs: ConfigValue) -> None:
 
 
 class Color:
-    """Represent an ANSI escape sequence """
+    """Represent an ANSI escape sequence"""
 
     def __init__(self, name: str, code: str):
         self.name = name
@@ -122,7 +124,7 @@ darkyellow = yellow
 
 # Other nice-to-have characters:
 class UnicodeSequence:
-    """ Represent a sequence containing a color followed by a Unicode symbol """
+    """Represent a sequence containing a color followed by a Unicode symbol"""
 
     def __init__(self, color: Color, as_unicode: str, as_ascii: str):
         if os.name == "nt":
@@ -178,7 +180,7 @@ def write_title_string(mystr: str, fileobj: FileObj) -> None:
 def process_tokens(
     tokens: Sequence[Token], *, end: str = "\n", sep: str = " "
 ) -> Tuple[str, str]:
-    """ Returns two strings from a list of tokens.
+    """Returns two strings from a list of tokens.
     One containing ASCII escape codes, the other
     only the 'normal' characters
 
@@ -240,9 +242,7 @@ def message(
     fileobj: FileObj = sys.stdout,
     update_title: bool = False,
 ) -> None:
-    """ Helper method for error, warning, info, debug
-
-    """
+    """Helper method for error, warning, info, debug"""
     should_use_colors = colors_enabled(fileobj)
     with_color, without_color = process_tokens(tokens, end=end, sep=sep)
     if CONFIG["record"]:
@@ -254,27 +254,27 @@ def message(
 
 
 def fatal(*tokens: Token, **kwargs: Any) -> None:
-    """ Print an error message and call ``sys.exit`` """
+    """Print an error message and call ``sys.exit``"""
     error(*tokens, **kwargs)
     sys.exit(1)
 
 
 def error(*tokens: Token, **kwargs: Any) -> None:
-    """ Print an error message """
+    """Print an error message"""
     tokens = [bold, red, "Error:"] + list(tokens)  # type: ignore
     kwargs["fileobj"] = sys.stderr
     message(*tokens, **kwargs)
 
 
 def warning(*tokens: Token, **kwargs: Any) -> None:
-    """ Print a warning message """
+    """Print a warning message"""
     tokens = [brown, "Warning:"] + list(tokens)  # type: ignore
     kwargs["fileobj"] = sys.stderr
     message(*tokens, **kwargs)
 
 
 def info(*tokens: Token, **kwargs: Any) -> None:
-    r""" Print an informative message
+    r"""Print an informative message
 
     :param tokens: list of `ui` constants or strings, like ``(cli_ui.red, 'this is an error')``
     :param sep: separator, defaults to ``' '``
@@ -288,7 +288,7 @@ def info(*tokens: Token, **kwargs: Any) -> None:
 
 
 def info_section(*tokens: Token, **kwargs: Any) -> None:
-    """ Print an underlined section name """
+    """Print an underlined section name"""
     # We need to know the length of the section:
     process_tokens_kwargs = kwargs.copy()
     process_tokens_kwargs["color"] = False
@@ -298,22 +298,22 @@ def info_section(*tokens: Token, **kwargs: Any) -> None:
 
 
 def info_1(*tokens: Token, **kwargs: Any) -> None:
-    """ Print an important informative message """
+    """Print an important informative message"""
     info(bold, blue, "::", reset, *tokens, **kwargs)
 
 
 def info_2(*tokens: Token, **kwargs: Any) -> None:
-    """ Print an not so important informative message """
+    """Print an not so important informative message"""
     info(bold, blue, "=>", reset, *tokens, **kwargs)
 
 
 def info_3(*tokens: Token, **kwargs: Any) -> None:
-    """ Print an even less important informative message """
+    """Print an even less important informative message"""
     info(bold, blue, "*", reset, *tokens, **kwargs)
 
 
 def dot(*, last: bool = False, fileobj: FileObj = sys.stdout) -> None:
-    """ Print a dot without a newline unless it is the last one.
+    """Print a dot without a newline unless it is the last one.
 
     Useful when you want to display a progress with very little
     knowledge.
@@ -325,7 +325,7 @@ def dot(*, last: bool = False, fileobj: FileObj = sys.stdout) -> None:
 
 
 def info_count(i: int, n: int, *rest: Token, **kwargs: Any) -> None:
-    """ Display a counter before the rest of the message.
+    """Display a counter before the rest of the message.
 
     ``rest`` and ``kwargs`` are passed to :func:`info`
 
@@ -341,7 +341,7 @@ def info_count(i: int, n: int, *rest: Token, **kwargs: Any) -> None:
 
 
 def info_progress(prefix: str, value: float, max_value: float) -> None:
-    """ Display info progress in percent.
+    """Display info progress in percent.
 
     :param value: the current value
     :param max_value: the max value
@@ -349,14 +349,14 @@ def info_progress(prefix: str, value: float, max_value: float) -> None:
 
 
     """
-    if sys.stdout.isatty():
+    if colors_enabled(sys.stdout):
         percent = float(value) / max_value * 100
-        sys.stdout.write(prefix + ": %.0f%%\r" % percent)
-        sys.stdout.flush()
+        to_write = prefix + ": %.0f%%\r" % percent
+        write_and_flush(sys.stdout, to_write)
 
 
 def debug(*tokens: Token, **kwargs: Any) -> None:
-    """ Print a debug message.
+    """Print a debug message.
 
     Messages are shown only when ``CONFIG["verbose"]`` is true
     """
@@ -367,7 +367,7 @@ def debug(*tokens: Token, **kwargs: Any) -> None:
 
 def indent_iterable(elems: Sequence[str], num: int = 2) -> List[str]:
     """Indent an iterable."""
-    return [" " * num + l for l in elems]
+    return [" " * num + elem for elem in elems]
 
 
 def indent(text: str, num: int = 2) -> str:
@@ -377,9 +377,7 @@ def indent(text: str, num: int = 2) -> str:
 
 
 def tabs(num: int) -> str:
-    """ Compute a blank tab
-
-    """
+    """Compute a blank tab"""
     return "  " * num
 
 
@@ -422,7 +420,7 @@ def info_table(
 
 
 def message_for_exception(exception: Exception, message: str) -> Sequence[Token]:
-    """ Returns a tuple suitable for cli_ui.error()
+    """Returns a tuple suitable for cli_ui.error()
     from the given exception.
     (Traceback will be part of the message, after
     the ``message`` argument)
@@ -445,17 +443,13 @@ def message_for_exception(exception: Exception, message: str) -> Sequence[Token]
 
 
 def read_input() -> str:
-    """ Read input from the user
-
-    """
+    """Read input from the user"""
     info(green, "> ", end="")
     return input()
 
 
 def read_password() -> str:
-    """ Read a password from the user
-
-    """
+    """Read a password from the user"""
     info(green, "> ", end="")
     return getpass.getpass(prompt="")
 
@@ -465,8 +459,7 @@ def get_ask_tokens(tokens: Sequence[Token]) -> List[Token]:
 
 
 def ask_string(*question: Token, default: Optional[str] = None) -> Optional[str]:
-    """Ask the user to enter a string.
-    """
+    """Ask the user to enter a string."""
     tokens = get_ask_tokens(question)
     if default:
         tokens.append("(%s)" % default)
@@ -478,8 +471,7 @@ def ask_string(*question: Token, default: Optional[str] = None) -> Optional[str]
 
 
 def ask_password(*question: Token) -> str:
-    """Ask the user to enter a password.
-    """
+    """Ask the user to enter a password."""
     tokens = get_ask_tokens(question)
     info(*tokens)
     answer = read_password()
@@ -565,9 +557,7 @@ AnyFunc = Callable[..., Any]
 
 
 class Timer:
-    """ Display time taken when executing a list of statements.
-
-    """
+    """Display time taken when executing a list of statements."""
 
     def __init__(self, description: str):
         self.description = description
@@ -593,11 +583,11 @@ class Timer:
         self.stop()
 
     def start(self) -> None:
-        """ Start the timer """
+        """Start the timer"""
         self.start_time = datetime.datetime.now()
 
     def stop(self) -> None:
-        """ Stop the timer and emit a nice log """
+        """Stop the timer and emit a nice log"""
         end_time = datetime.datetime.now()
         elapsed_time = end_time - self.start_time
         elapsed_seconds = elapsed_time.seconds
@@ -613,7 +603,7 @@ class Timer:
 
 
 def did_you_mean(message: str, user_input: str, choices: Sequence[str]) -> str:
-    """ Given a list of choices and an invalid user input, display the closest
+    """Given a list of choices and an invalid user input, display the closest
     items in the list that match the input.
 
     """
@@ -636,44 +626,43 @@ def main_test_colors() -> None:
 
 
 def main_demo() -> None:
+    info_section(bold, "python-cli demo")
+    info("This is", red, "red")
+    info("this is", bold, "bold")
     info("OK", check)
     up = Symbol("👍", "+1")
     info("I like it", blue, up)
-    info_section(bold, "python-cli demo")
-    # Monkey-patch message() so that we sleep after
-    # each call
-    global message
-    old_message = message
-
-    def new_message(*args: Any, **kwargs: Any) -> None:
-        old_message(*args, **kwargs)
-        time.sleep(0.5)
-
-    message = new_message
 
+    info()
+    info_section(bold, "various levels of messages")
     info_1("Important info")
     info_2("Secondary info")
-    info("This is", red, "red")
-    info("this is", bold, "bold")
+    info_3("More detailed info")
+
+    info()
+    info_section(bold, "progress info")
+
     list_of_things = ["foo", "bar", "baz"]
     for i, thing in enumerate(list_of_things):
         info_count(i, len(list_of_things), thing)
-    info_progress("Done", 5, 20)
-    info_progress("Done", 10, 20)
-    info_progress("Done", 20, 20)
-    info("\n", check, "all done")
+    info()
 
-    # stop monkey patching
-    message = old_message
-    fruits = ["apple", "orange", "banana"]
-    answer = ask_choice("Choose a fruit", choices=fruits)
-    info("You chose:", answer)
+    time.sleep(0.5)
+    info_progress("Doing something", 5, 20)
+    time.sleep(0.5)
+    info_progress("Doing something", 10, 20)
+    time.sleep(0.5)
+    info_progress("Doing something", 20, 20)
+    time.sleep(0.5)
+    info("\n", check, "all done")
 
 
 def main() -> None:
     parser = argparse.ArgumentParser()
+    parser.add_argument("--color", choices=["always", "never", "auto"])
     parser.add_argument("action", choices=["test_colors", "demo"])
     args = parser.parse_args()
+    setup(color=args.color)
     if args.action == "demo":
         main_demo()
     elif args.action == "test_colors":
